@@ -1,41 +1,41 @@
 from __future__ import annotations
 
+import errno
+import os
 import socket
 import ssl
+import stat
 import sys
+from collections.abc import Awaitable
 from ipaddress import IPv6Address, ip_address
 from os import PathLike, chmod
-from pathlib import Path
 from socket import AddressFamily, SocketKind
-from typing import Awaitable, List, Tuple, cast, overload
+from typing import Any, Literal, cast, overload
 
 from .. import to_thread
 from ..abc import (
     ConnectedUDPSocket,
+    ConnectedUNIXDatagramSocket,
     IPAddressType,
     IPSockAddrType,
     SocketListener,
     SocketStream,
     UDPSocket,
+    UNIXDatagramSocket,
     UNIXSocketStream,
 )
 from ..streams.stapled import MultiListener
 from ..streams.tls import TLSStream
-from ._eventloop import get_asynclib
+from ._eventloop import get_async_backend
 from ._resources import aclose_forcefully
 from ._synchronization import Event
 from ._tasks import create_task_group, move_on_after
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+if sys.version_info < (3, 11):
+    from exceptiongroup import ExceptionGroup
 
 IPPROTO_IPV6 = getattr(socket, "IPPROTO_IPV6", 41)  # https://bugs.python.org/issue29515
 
-GetAddrInfoReturnType = List[
-    Tuple[AddressFamily, SocketKind, int, str, Tuple[str, int]]
-]
 AnyIPAddressFamily = Literal[
     AddressFamily.AF_UNSPEC, AddressFamily.AF_INET, AddressFamily.AF_INET6
 ]
@@ -142,18 +142,21 @@ async def connect_tcp(
 
     :param remote_host: the IP address or host name to connect to
     :param remote_port: port on the target host to connect to
-    :param local_host: the interface address or name to bind the socket to before connecting
+    :param local_host: the interface address or name to bind the socket to before
+        connecting
     :param tls: ``True`` to do a TLS handshake with the connected stream and return a
         :class:`~anyio.streams.tls.TLSStream` instead
-    :param ssl_context: the SSL context object to use (if omitted, a default context is created)
-    :param tls_standard_compatible: If ``True``, performs the TLS shutdown handshake before closing
-        the stream and requires that the server does this as well. Otherwise,
-        :exc:`~ssl.SSLEOFError` may be raised during reads from the stream.
+    :param ssl_context: the SSL context object to use (if omitted, a default context is
+        created)
+    :param tls_standard_compatible: If ``True``, performs the TLS shutdown handshake
+        before closing the stream and requires that the server does this as well.
+        Otherwise, :exc:`~ssl.SSLEOFError` may be raised during reads from the stream.
         Some protocols, such as HTTP, require this option to be ``False``.
         See :meth:`~ssl.SSLContext.wrap_socket` for details.
-    :param tls_hostname: host name to check the server certificate against (defaults to the value
-        of ``remote_host``)
-    :param happy_eyeballs_delay: delay (in seconds) before starting the next connection attempt
+    :param tls_hostname: host name to check the server certificate against (defaults to
+        the value of ``remote_host``)
+    :param happy_eyeballs_delay: delay (in seconds) before starting the next connection
+        attempt
     :return: a socket stream object if no TLS handshake was done, otherwise a TLS stream
     :raises OSError: if the connection attempt fails
 
@@ -177,7 +180,7 @@ async def connect_tcp(
         finally:
             event.set()
 
-    asynclib = get_asynclib()
+    asynclib = get_async_backend()
     local_address: IPSockAddrType | None = None
     family = socket.AF_UNSPEC
     if local_host:
@@ -193,8 +196,8 @@ async def connect_tcp(
             target_host, remote_port, family=family, type=socket.SOCK_STREAM
         )
 
-        # Organize the list so that the first address is an IPv6 address (if available) and the
-        # second one is an IPv4 addresses. The rest can be in whatever order.
+        # Organize the list so that the first address is an IPv6 address (if available)
+        # and the second one is an IPv4 addresses. The rest can be in whatever order.
         v6_found = v4_found = False
         target_addrs: list[tuple[socket.AddressFamily, str]] = []
         for af, *rest, sa in gai_res:
@@ -221,7 +224,11 @@ async def connect_tcp(
                 await event.wait()
 
     if connected_stream is None:
-        cause = oserrors[0] if len(oserrors) == 1 else asynclib.ExceptionGroup(oserrors)
+        cause = (
+            oserrors[0]
+            if len(oserrors) == 1
+            else ExceptionGroup("multiple connection attempts failed", oserrors)
+        )
         raise OSError("All connection attempts failed") from cause
 
     if tls or tls_hostname or ssl_context:
@@ -240,7 +247,7 @@ async def connect_tcp(
     return connected_stream
 
 
-async def connect_unix(path: str | PathLike[str]) -> UNIXSocketStream:
+async def connect_unix(path: str | bytes | PathLike[Any]) -> UNIXSocketStream:
     """
     Connect to the given UNIX socket.
 
@@ -250,8 +257,8 @@ async def connect_unix(path: str | PathLike[str]) -> UNIXSocketStream:
     :return: a socket stream object
 
     """
-    path = str(Path(path))
-    return await get_asynclib().connect_unix(path)
+    path = os.fspath(path)
+    return await get_async_backend().connect_unix(path)
 
 
 async def create_tcp_listener(
@@ -277,11 +284,11 @@ async def create_tcp_listener(
     :return: a list of listener objects
 
     """
-    asynclib = get_asynclib()
+    asynclib = get_async_backend()
     backlog = min(backlog, 65536)
     local_host = str(local_host) if local_host is not None else None
     gai_res = await getaddrinfo(
-        local_host,  # type: ignore[arg-type]
+        local_host,
         local_port,
         family=family,
         type=socket.SocketKind.SOCK_STREAM if sys.platform == "win32" else 0,
@@ -302,7 +309,8 @@ async def create_tcp_listener(
             raw_socket = socket.socket(fam)
             raw_socket.setblocking(False)
 
-            # For Windows, enable exclusive address use. For others, enable address reuse.
+            # For Windows, enable exclusive address use. For others, enable address
+            # reuse.
             if sys.platform == "win32":
                 raw_socket.setsockopt(socket.SOL_SOCKET, socket.SO_EXCLUSIVEADDRUSE, 1)
             else:
@@ -322,7 +330,7 @@ async def create_tcp_listener(
 
             raw_socket.bind(sockaddr)
             raw_socket.listen(backlog)
-            listener = asynclib.TCPSocketListener(raw_socket)
+            listener = asynclib.create_tcp_listener(raw_socket)
             listeners.append(listener)
     except BaseException:
         for listener in listeners:
@@ -334,7 +342,7 @@ async def create_tcp_listener(
 
 
 async def create_unix_listener(
-    path: str | PathLike[str],
+    path: str | bytes | PathLike[Any],
     *,
     mode: int | None = None,
     backlog: int = 65536,
@@ -346,29 +354,20 @@ async def create_unix_listener(
 
     :param path: path of the socket
     :param mode: permissions to set on the socket
-    :param backlog: maximum number of queued incoming connections (up to a maximum of 2**16, or
-        65536)
+    :param backlog: maximum number of queued incoming connections (up to a maximum of
+        2**16, or 65536)
     :return: a listener object
 
     .. versionchanged:: 3.0
-        If a socket already exists on the file system in the given path, it will be removed first.
+        If a socket already exists on the file system in the given path, it will be
+        removed first.
 
     """
-    path_str = str(path)
-    path = Path(path)
-    if path.is_socket():
-        path.unlink()
-
     backlog = min(backlog, 65536)
-    raw_socket = socket.socket(socket.AF_UNIX)
-    raw_socket.setblocking(False)
+    raw_socket = await setup_unix_local_socket(path, mode, socket.SOCK_STREAM)
     try:
-        await to_thread.run_sync(raw_socket.bind, path_str, cancellable=True)
-        if mode is not None:
-            await to_thread.run_sync(chmod, path_str, mode, cancellable=True)
-
         raw_socket.listen(backlog)
-        return get_asynclib().UNIXSocketListener(raw_socket)
+        return get_async_backend().create_unix_listener(raw_socket)
     except BaseException:
         raw_socket.close()
         raise
@@ -384,15 +383,15 @@ async def create_udp_socket(
     """
     Create a UDP socket.
 
-    If ``local_port`` has been given, the socket will be bound to this port on the local
+    If ``port`` has been given, the socket will be bound to this port on the local
     machine, making this socket suitable for providing UDP based services.
 
-    :param family: address family (``AF_INET`` or ``AF_INET6``) – automatically determined from
-        ``local_host`` if omitted
+    :param family: address family (``AF_INET`` or ``AF_INET6``) – automatically
+        determined from ``local_host`` if omitted
     :param local_host: IP address or host name of the local interface to bind to
     :param local_port: local port to bind to
-    :param reuse_port: ``True`` to allow multiple sockets to bind to the same address/port
-        (not supported on Windows)
+    :param reuse_port: ``True`` to allow multiple sockets to bind to the same
+        address/port (not supported on Windows)
     :return: a UDP socket
 
     """
@@ -414,9 +413,10 @@ async def create_udp_socket(
     else:
         local_address = ("0.0.0.0", 0)
 
-    return await get_asynclib().create_udp_socket(
+    sock = await get_async_backend().create_udp_socket(
         family, local_address, None, reuse_port
     )
+    return cast(UDPSocket, sock)
 
 
 async def create_connected_udp_socket(
@@ -431,17 +431,17 @@ async def create_connected_udp_socket(
     """
     Create a connected UDP socket.
 
-    Connected UDP sockets can only communicate with the specified remote host/port, and any packets
-    sent from other sources are dropped.
+    Connected UDP sockets can only communicate with the specified remote host/port, an
+    any packets sent from other sources are dropped.
 
     :param remote_host: remote host to set as the default target
     :param remote_port: port on the remote host to set as the default target
-    :param family: address family (``AF_INET`` or ``AF_INET6``) – automatically determined from
-        ``local_host`` or ``remote_host`` if omitted
+    :param family: address family (``AF_INET`` or ``AF_INET6``) – automatically
+        determined from ``local_host`` or ``remote_host`` if omitted
     :param local_host: IP address or host name of the local interface to bind to
     :param local_port: local port to bind to
-    :param reuse_port: ``True`` to allow multiple sockets to bind to the same address/port
-        (not supported on Windows)
+    :param reuse_port: ``True`` to allow multiple sockets to bind to the same
+        address/port (not supported on Windows)
     :return: a connected UDP socket
 
     """
@@ -463,25 +463,87 @@ async def create_connected_udp_socket(
     family = cast(AnyIPAddressFamily, gai_res[0][0])
     remote_address = gai_res[0][-1]
 
-    return await get_asynclib().create_udp_socket(
+    sock = await get_async_backend().create_udp_socket(
         family, local_address, remote_address, reuse_port
     )
+    return cast(ConnectedUDPSocket, sock)
+
+
+async def create_unix_datagram_socket(
+    *,
+    local_path: None | str | bytes | PathLike[Any] = None,
+    local_mode: int | None = None,
+) -> UNIXDatagramSocket:
+    """
+    Create a UNIX datagram socket.
+
+    Not available on Windows.
+
+    If ``local_path`` has been given, the socket will be bound to this path, making this
+    socket suitable for receiving datagrams from other processes. Other processes can
+    send datagrams to this socket only if ``local_path`` is set.
+
+    If a socket already exists on the file system in the ``local_path``, it will be
+    removed first.
+
+    :param local_path: the path on which to bind to
+    :param local_mode: permissions to set on the local socket
+    :return: a UNIX datagram socket
+
+    """
+    raw_socket = await setup_unix_local_socket(
+        local_path, local_mode, socket.SOCK_DGRAM
+    )
+    return await get_async_backend().create_unix_datagram_socket(raw_socket, None)
+
+
+async def create_connected_unix_datagram_socket(
+    remote_path: str | bytes | PathLike[Any],
+    *,
+    local_path: None | str | bytes | PathLike[Any] = None,
+    local_mode: int | None = None,
+) -> ConnectedUNIXDatagramSocket:
+    """
+    Create a connected UNIX datagram socket.
+
+    Connected datagram sockets can only communicate with the specified remote path.
+
+    If ``local_path`` has been given, the socket will be bound to this path, making
+    this socket suitable for receiving datagrams from other processes. Other processes
+    can send datagrams to this socket only if ``local_path`` is set.
+
+    If a socket already exists on the file system in the ``local_path``, it will be
+    removed first.
+
+    :param remote_path: the path to set as the default target
+    :param local_path: the path on which to bind to
+    :param local_mode: permissions to set on the local socket
+    :return: a connected UNIX datagram socket
+
+    """
+    remote_path = os.fspath(remote_path)
+    raw_socket = await setup_unix_local_socket(
+        local_path, local_mode, socket.SOCK_DGRAM
+    )
+    return await get_async_backend().create_unix_datagram_socket(
+        raw_socket, remote_path
+    )
 
 
 async def getaddrinfo(
-    host: bytearray | bytes | str,
+    host: bytes | str | None,
     port: str | int | None,
     *,
     family: int | AddressFamily = 0,
     type: int | SocketKind = 0,
     proto: int = 0,
     flags: int = 0,
-) -> GetAddrInfoReturnType:
+) -> list[tuple[AddressFamily, SocketKind, int, str, tuple[str, int]]]:
     """
     Look up a numeric IP address given a host name.
 
-    Internationalized domain names are translated according to the (non-transitional) IDNA 2008
-    standard.
+    Internationalized domain names are translated according to the (non-transitional)
+    IDNA 2008 standard.
 
     .. note:: 4-tuple IPv6 socket addresses are automatically converted to 2-tuples of
         (host, port), unlike what :func:`socket.getaddrinfo` does.
@@ -500,7 +562,7 @@ async def getaddrinfo(
     # Handle unicode hostnames
     if isinstance(host, str):
         try:
-            encoded_host = host.encode("ascii")
+            encoded_host: bytes | None = host.encode("ascii")
         except UnicodeEncodeError:
             import idna
 
@@ -508,7 +570,7 @@ async def getaddrinfo(
     else:
         encoded_host = host
 
-    gai_res = await get_asynclib().getaddrinfo(
+    gai_res = await get_async_backend().getaddrinfo(
         encoded_host, port, family=family, type=type, proto=proto, flags=flags
     )
     return [
@@ -528,18 +590,18 @@ def getnameinfo(sockaddr: IPSockAddrType, flags: int = 0) -> Awaitable[tuple[str
     .. seealso:: :func:`socket.getnameinfo`
 
     """
-    return get_asynclib().getnameinfo(sockaddr, flags)
+    return get_async_backend().getnameinfo(sockaddr, flags)
 
 
 def wait_socket_readable(sock: socket.socket) -> Awaitable[None]:
     """
     Wait until the given socket has data to be read.
 
-    This does **NOT** work on Windows when using the asyncio backend with a proactor event loop
-    (default on py3.8+).
+    This does **NOT** work on Windows when using the asyncio backend with a proactor
+    event loop (default on py3.8+).
 
-    .. warning:: Only use this on raw sockets that have not been wrapped by any higher level
-        constructs like socket streams!
+    .. warning:: Only use this on raw sockets that have not been wrapped by any higher
+        level constructs like socket streams!
 
     :param sock: a socket object
     :raises ~anyio.ClosedResourceError: if the socket was closed while waiting for the
@@ -548,18 +610,18 @@ def wait_socket_readable(sock: socket.socket) -> Awaitable[None]:
         to become readable
 
     """
-    return get_asynclib().wait_socket_readable(sock)
+    return get_async_backend().wait_socket_readable(sock)
 
 
 def wait_socket_writable(sock: socket.socket) -> Awaitable[None]:
     """
     Wait until the given socket can be written to.
 
-    This does **NOT** work on Windows when using the asyncio backend with a proactor event loop
-    (default on py3.8+).
+    This does **NOT** work on Windows when using the asyncio backend with a proactor
+    event loop (default on py3.8+).
 
-    .. warning:: Only use this on raw sockets that have not been wrapped by any higher level
-        constructs like socket streams!
+    .. warning:: Only use this on raw sockets that have not been wrapped by any higher
+        level constructs like socket streams!
 
     :param sock: a socket object
     :raises ~anyio.ClosedResourceError: if the socket was closed while waiting for the
@@ -568,7 +630,7 @@ def wait_socket_writable(sock: socket.socket) -> Awaitable[None]:
         to become writable
 
     """
-    return get_asynclib().wait_socket_writable(sock)
+    return get_async_backend().wait_socket_writable(sock)
 
 
 #
@@ -577,7 +639,7 @@ def wait_socket_writable(sock: socket.socket) -> Awaitable[None]:
 
 
 def convert_ipv6_sockaddr(
-    sockaddr: tuple[str, int, int, int] | tuple[str, int]
+    sockaddr: tuple[str, int, int, int] | tuple[str, int],
 ) -> tuple[str, int]:
     """
     Convert a 4-tuple IPv6 socket address to a 2-tuple (address, port) format.
@@ -592,7 +654,7 @@ def convert_ipv6_sockaddr(
     """
     # This is more complicated than it should be because of MyPy
     if isinstance(sockaddr, tuple) and len(sockaddr) == 4:
-        host, port, flowinfo, scope_id = cast(Tuple[str, int, int, int], sockaddr)
+        host, port, flowinfo, scope_id = sockaddr
         if scope_id:
             # PyPy (as of v7.3.11) leaves the interface name in the result, so
             # we discard it and only get the scope ID from the end
@@ -604,4 +666,51 @@ def convert_ipv6_sockaddr(
         else:
             return host, port
     else:
-        return cast(Tuple[str, int], sockaddr)
+        return sockaddr
+
+
+async def setup_unix_local_socket(
+    path: None | str | bytes | PathLike[Any],
+    mode: int | None,
+    socktype: int,
+) -> socket.socket:
+    """
+    Create a UNIX local socket object, deleting the socket at the given path if it
+    exists.
+
+    Not available on Windows.
+
+    :param path: path of the socket
+    :param mode: permissions to set on the socket
+    :param socktype: socket.SOCK_STREAM or socket.SOCK_DGRAM
+
+    """
+    path_str: str | bytes | None
+    if path is not None:
+        path_str = os.fspath(path)
+
+        # Copied from pathlib...
+        try:
+            stat_result = os.stat(path)
+        except OSError as e:
+            if e.errno not in (errno.ENOENT, errno.ENOTDIR, errno.EBADF, errno.ELOOP):
+                raise
+        else:
+            if stat.S_ISSOCK(stat_result.st_mode):
+                os.unlink(path)
+    else:
+        path_str = None
+
+    raw_socket = socket.socket(socket.AF_UNIX, socktype)
+    raw_socket.setblocking(False)
+
+    if path_str is not None:
+        try:
+            await to_thread.run_sync(raw_socket.bind, path_str, abandon_on_cancel=True)
+            if mode is not None:
+                await to_thread.run_sync(chmod, path_str, mode, abandon_on_cancel=True)
+        except BaseException:
+            raw_socket.close()
+            raise
+
+    return raw_socket
